clear all
capture log close
set more off
set matsize 500
set linesize 255

** Jae: You should change the below directory. 
global maindirectory ="/Users/rceyfk01/Dropbox/SSA-INCOME-RISK/Fatih_trial_replication/Stata"
global unix=1  // JAE: Please change this to 1 if you run stata on Unix or Mac

** YOU DON'T NEED TO MAKE ANY CHANGES FROM THIS POINT ON!!!!

if($unix==1){
	global sep="/"
}
else{
	global sep="\"
}
global outfolder=c(current_date)
global outfolder="$outfolder LifeCycleProfile_LABOR"
capture noisily mkdir "$maindirectory${sep}out${sep}$outfolder"
cd "$maindirectory${sep}out${sep}$outfolder"

do "$maindirectory${sep}do${sep}bymydiststat2.do"
do "$maindirectory${sep}do${sep}bymyxtile.do"

global logname="$maindirectory${sep}log${sep}$outfolder${sep}$outfolder.log"
capture noisily log close
capture noisily log using "$logname", replace

global yrfirst = 1978 		// First year in the dataset
global yrlast  = 2013 		// Last year in the dataset
global begin_age  = 25
global retire_age = 60
global varoi="labor" 		// Income list that we analyze
global statlist="mean sd skew kurt kurt2 min max p10 p50 p90"

global begin=c(current_time)

global base_price = 52   /* this is the index of the year 2010 */
matrix cpimat = /*  CPI for 1959-2013
*/ (17.262,17.546,17.730,17.939,18.149,18.414,18.681,19.155,19.637,20.402,	/*
*/ 21.327,22.325,23.274,24.070,25.368,28.009,30.348,32.013,34.091,36.479, 	/*
*/ 39.714,43.978,47.908,50.553,52.729,54.724,56.661,57.887,59.650,61.974,	/*
*/ 64.641,64.641,67.440,69.652,71.494,73.279,74.803,76.356,77.981,79.327,	/*
*/ 79.936,81.110,83.131,84.736,85.873,87.572,89.703,92.261,94.729,97.101, 	/*
*/ 100.065,100.000,101.653,104.149,106.062,107.333)

matrix minwg = /* Nominal minimum wage 1959-2013
*/ (1.00,1.00,1.00,1.15,1.15,1.25,1.25,1.25,1.25,1.40,1.60,1.60,1.60,1.60,/*
*/  1.60,1.60,2.00,2.10,2.10,2.30,2.65,2.90,3.10,3.35,3.35,3.35,3.35,3.35,/*
*/  3.35,3.35,3.35,3.35,3.80,4.25,4.25,4.25,4.25,4.25,4.75,5.15,5.15,5.15,/*
*/  5.15,5.15,5.15,5.15,5.15,5.15,5.15,5.85,6.55,7.25,7.25,7.25,7.25)

matrix rmininc = J(rowsof(minwg),colsof(minwg), 0.0)
forvalues yr = 1959/2013{
	local cpi_index = `yr'-1959+1
	local deflate = cpimat[1,${base_price}]/cpimat[1,`cpi_index']
	matrix rmininc[1,`cpi_index']=40*13*0.5*minwg[1,`cpi_index']*`deflate'
}

** Individual Statistics
use  "$maindirectory${sep}dta${sep}cleaned_MALE_10p_LABOR2013", clear

** Sample selection & lifetime income construction
** 1) Require at least 32 income observations
** 2) Require at least 15 income observations above the threshold
gen yrobs = min(${yrlast}-max(${yrlast}-yob+1-${retire_age},0),yod-1) - ///
			 (${yrfirst}-min(${yrfirst}-yob+1-${begin_age},0)) + 1
drop if yrobs<33
tab yob
gen ${varoi}emp=0
gen LT${varoi}=0
gen lifetime=0
forvalues yr = $yrfirst/$yrlast{
	qui replace ${varoi}emp=${varoi}emp+1 if 	///
		${varoi}`yr'> rmininc[1,`yr'-1959+1] & ${varoi}`yr'~=.
	qui replace LT${varoi}=LT${varoi}+${varoi}`yr' if ${varoi}`yr'~=.
	qui replace lifetime=lifetime+1 if ${varoi}`yr'~=.
}
tab ${varoi}emp
drop if ${varoi}emp<15
tab ${varoi}emp

replace LT${varoi}=LT${varoi}/lifetime
drop yrobs lifetime

gen id=_n
save "$maindirectory${sep}dta${sep}LifeCycleProfileData_LABOR.dta", replace
xtile incrank=LT${varoi}, nq(100)
qui gen age=.
*foreach age in 25 30 35 40 45 50 55 60{
forvalues age=25/60{
	qui gen temp${varoi}=.
	qui replace age=`age'
	forvalues coh=1951/1957{
		local year=`age'+`coh'-1
		if `year'>=	$yrfirst & `year' <=$yrlast{
			qui replace temp${varoi}=${varoi}`year' if yob==`coh'
		}
	}
	bymydiststat "temp${varoi}" "D_" "`age'" "age incrank"
	drop temp${varoi}
}

*foreach age in 25 30 35 40 45 50 55 60{
forvalues age=25/60{
	drop _all 
	use D_temp${varoi}`age'.dta
	foreach stat in N mean sd skew kurt min max p1 p5 p10 p25 p50 p75 p90 p95 p99{
		rename D_`stat'temp${varoi}`age' `stat'${varoi}
	}
	save D_temp${varoi}`age'.dta, replace
}
drop _all
set obs 1
gen tobedropped=1
*foreach age in 25 30 35 40 45 50 55 60{
forvalues age=25/60{
	append using D_temp${varoi}`age'.dta
	erase D_temp${varoi}`age'.dta
}
drop if tobedropped==1 | age==.
drop tobedropped
qui outsheet using LifeCycleProfile_LABOR.txt, replace

capture log close
